package org.netbeans.demo.replace.property.type.nodes;
import java.lang.reflect.InvocationTargetException;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.PropertySupport.ReadWrite;
import org.openide.nodes.Sheet;

public class MyNode extends AbstractNode {
  private boolean firstType = true;
  private FirstEnum firstEnumVal = FirstEnum.FIRST_ONE;
  private SecondEnum secondEnumVal = SecondEnum.SECOND_ONE;
  //Create our property sheet ahead of time and cache
  private final Sheet sheet = Sheet.createDefault();
  //Same for the property set
  private final Sheet.Set propertySet = Sheet.createPropertiesSet();
  public MyNode() {
    super(Children.LEAF);
    setDisplayName("See My Properties");
    //Add the one property set to the sheet - no need to do this again
    sheet.put(propertySet);
  }

  @Override
  protected Sheet createSheet() {
    propertySet.put(new ChangeOtherPropertysTypeProp());
    //add our mutable property
    updateEnumProp();
    return sheet;
  }

  private void updateEnumProp() {
    //Rely on the Propertys having the same *name* to ensure one
    //replaces the other
    propertySet.put(firstType ? new FirstEnumProp() : new SecondEnumProp());
  }

  public boolean isFirstType() {
    return firstType;
  }

  private final class ChangeOtherPropertysTypeProp extends
                                          ReadWrite<Boolean> {
    public ChangeOtherPropertysTypeProp() {
      super("firstType", Boolean.class, "Change type", null);
    }

    @Override
    public Boolean getValue() throws IllegalAccessException,
                                     InvocationTargetException {
      return firstType;
    }

    @Override
    public void setValue(Boolean nue) throws IllegalAccessException,
                                             IllegalArgumentException,
                                             InvocationTargetException {
      boolean old = firstType;
      if (nue.booleanValue() != old) {
        firstType = nue;
        updateEnumProp();
        MyNode.this.firePropertyChange("firstType", old, nue);
      }
    }
  }
  //The rest is boilerplate
  private final class FirstEnumProp extends ReadWrite<FirstEnum> {
    FirstEnumProp() {
      super("enumProp", FirstEnum.class, "Enum Prop", null);
    }

    @Override
    public FirstEnum getValue() throws IllegalAccessException,
                                       InvocationTargetException {
      return firstEnumVal;
    }

    @Override
    public void setValue(FirstEnum nue) throws IllegalAccessException,
                                               IllegalArgumentException,
                                               InvocationTargetException {
      if (nue != firstEnumVal) {
        FirstEnum old = firstEnumVal;
        firstEnumVal = nue;
        firePropertyChange("enumProp", old, nue);
      }
    }
  }

  private final class SecondEnumProp extends ReadWrite<SecondEnum> {
    SecondEnumProp() {
      super("enumProp", SecondEnum.class, "Enum Prop", null);
    }

    @Override
    public SecondEnum getValue() throws IllegalAccessException,
                                        InvocationTargetException {
      return secondEnumVal;
    }

    @Override
    public void setValue(SecondEnum nue) throws IllegalAccessException,
                                                IllegalArgumentException,
                                                InvocationTargetException {
      if (nue != secondEnumVal) {
        SecondEnum old = secondEnumVal;
        secondEnumVal = nue;
        firePropertyChange("enumProp", old, nue);
      }
    }
  }
}
