package org.netbeans.demo.replace.property.type.ui;
import java.awt.BorderLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Arrays;
import org.netbeans.demo.replace.property.type.nodes.MyNode;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerUtils;
import org.openide.explorer.view.BeanTreeView;
import org.openide.explorer.view.OutlineView;
import org.openide.nodes.Node.PropertySet;
import org.openide.windows.TopComponent;

/**
 *
 * @author Tim Boudreau
 */
public class WindowCompTopComponent extends TopComponent implements ExplorerManager.Provider, PropertyChangeListener {
  private final ExplorerManager mgr = new ExplorerManager();
  private final BeanTreeView tree = new BeanTreeView();
  private final OutlineView outline = new OutlineView();
  private final MyNode root = new MyNode();

  public WindowCompTopComponent() {
    setLayout(new BorderLayout());
    add(tree, BorderLayout.CENTER);
    add(outline, BorderLayout.SOUTH);
    associateLookup(ExplorerUtils.createLookup(mgr, getActionMap()));
    mgr.setRootContext(root);
    updateOutlineProperties();
    
    //We need to listen for changes, because the list of Properties in the
    //Outline is sensitive to the exact type of the prototype property we
    //pass it, unfortunately
    root.addPropertyChangeListener(this);
  }
  
  public void propertyChange(PropertyChangeEvent evt) {
    if ("firstType".equals(evt.getPropertyName())) {
      updateOutlineProperties();
    }
  }

  private void updateOutlineProperties() {
    System.err.println("Updating outline properties");
    //do this
//    outline.setProperties(new Property[]{
//              new PrototypeProperty<Boolean>("firstType", Boolean.class),
//              new PrototypeProperty("enumProp", root.isFirstType() ? FirstEnum.class : SecondEnum.class)
//            });
    //or...
    PropertySet set = root.getPropertySets()[0];
    System.out.println("PROPERTIES: " + Arrays.asList(set.getProperties()));
    outline.setProperties(set.getProperties());
  }

  @Override
  public int getPersistenceType() {
    return PERSISTENCE_ONLY_OPENED;
  }

  public ExplorerManager getExplorerManager() {
    return mgr;
  }

  public static WindowCompTopComponent getDefault() {
    return new WindowCompTopComponent();
  }
}
